/* Name				: main.c
 * Project			: BCM Dynamic USB Device
 * Author			: Vitaliy Popov
 * Target MCU		: ATtiny2313
 * InterfaceClass	: HID
 * Data update		: 2009-10-26
 * Tabsize			: 4
 * Copyright		: (c) 2012 by OBJECTIVE DEVELOPMENT Software GmbH
 * License			: Proprietary, free under certain conditions. See GNU GPL v2, GNU GPL v3
 */

/*
         HID  
   AVR   8-     
 (BCM - Binary Code Modulation).      AVR  
.        , 
INT0     T/C0   CTC.      
I/O USB,    usbconfig.h.
*/

#include <avr/io.h>				//   -  
#include <avr/wdt.h>			//     
#include <avr/interrupt.h>		//     
#include <avr/pgmspace.h>		//     PROGMEM
#include <util/delay.h>			//     _delay_ms()

#include "usbdrv.h"

static const uchar bitMask[8] = {1,2,4,8,16,32,64,128};	//  

static uchar reportData[8];			//  HID 
static uchar bufferData[16];		//  
static uchar newData = 1;			//    , 1  
static uchar setPos  = 8;			//      (0,8)

static volatile uchar getPos = 0;	//      (0,8)
static volatile uchar bcmPos = 0;	//      (0,8)
static volatile uchar index  = 0;	//    (0..7,7..0)
static volatile uchar duplex = 0;	//    (0..15)


/* ------------------------------------------------------------------------- */
/* ----------------------------- HID  USB ---------------------------- */
/* ------------------------------------------------------------------------- */

/*
     .   
USB_CFG_HID_REPORT_DESCRIPTOR_LENGTH = 22  usbconfig.h
*/

const PROGMEM char usbHidReportDescriptor[USB_CFG_HID_REPORT_DESCRIPTOR_LENGTH] = {
	0x06, 0x00, 0xff,		// USAGE_PAGE (Vendor Defined Page 1)
	0x09, 0x01,				// USAGE (Vendor Usage 1)
	0xa1, 0x01,				// COLLECTION (Application)
	0x15, 0x00,					// LOGICAL_MINIMUM (0)
	0x26, 0xff, 0x00,			// LOGICAL_MAXIMUM (255)
	0x75, 0x08,					// REPORT_SIZE (8)
	0x95, 0x08,					// REPORT_COUNT (8)
	0x09, 0x00,					// USAGE (Undefined)
	0xb2, 0x02, 0x01,			// FEATURE (Data,Var,Abs,Buf)
	0xc0					// END_COLLECTION
};

/*
      feature-,     
report-ID,       .    
   .
*/


/* ------------------------------------------------------------------------- */
/* -----------------------------   ----------------------------- */
/* ------------------------------------------------------------------------- */

/*
  usbFunctionWrite() ,       .
      USB_CFG_IMPLEMENT_FN_WRITE = 1 
 usbconfig.h.    .   usbdrv/usbdrv.h.
*/

USB_PUBLIC uchar usbFunctionWrite(uchar *data, uchar len)
{
	uchar i;
	uchar *buffer = (uchar*) &reportData;	//     
	for(i = 0; i < len; i++)				//    len = REPORT_COUNT = 8 
		buffer[i] = data[i];				//     reportData
	newData = 1;							//  ,   
	return 1;								//   -   
}

/* ------------------------------------------------------------------------- */

/*
  usbFunctionSetup() ,     SETUP
 ,      ( :  
 ).       SETUP,   
  ()  .  SETUP   
'data',   ( )    'usbRequest_t *'  
     .    .
  usbdrv/usbdrv.h.
*/

USB_PUBLIC usbMsgLen_t usbFunctionSetup(uchar data[8])
{
	usbRequest_t *rq = (void *)data;
	if((rq->bmRequestType & USBRQ_TYPE_MASK) == USBRQ_TYPE_CLASS){ //   HID 
		if(rq->bRequest == USBRQ_HID_SET_REPORT){ // wValue: ReportType (highbyte), ReportID (lowbyte)
			//       ,    -ID
			newData = 0;		//  ,   
			return USB_NO_MSG;	//  usbFunctionWrite()      
		}else{
				//    USB HID,      
		 }
	}else{
			//    ,      
	 }
	return 0;
}


/* ------------------------------------------------------------------------- */
/* -----------------     ----------------- */
/* ------------------------------------------------------------------------- */

/*
       8-    
     8  .  
          
   .
*/

static uchar bcmMethod(void)
{
	uchar nPer;					// 8   ()
	uchar nBit;					// -   BCM
	uchar bufValue;				//    -   BCM
	uchar nBitMask;				//  
	uchar nPerMask = 1;			//  

	if (setPos == getPos)		//      ISR, 
		return 1;				// ,   ,   

	newData = 0;							//    
	for (nPer = 0; nPer <= 7; nPer++)		//  8  
	{
		bufValue = 0;						//  
		nBitMask = 1;						//  
		for (nBit = 0; nBit <= 7; nBit++)	//   
		{
			if (reportData[nBit] & nPerMask) bufValue |= nBitMask;	//  BCM
			nBitMask <<= 1;					//   	
		}
		bufferData[nPer+setPos] = bufValue;	//    
		nPerMask <<= 1;						//   
	}

	if (newData == 0)			//         , 
	{
		bcmPos = setPos;		//        ISR
		setPos = 8-setPos;		//      
	}

	return newData;				//    
}

/* --------------------     -------------------- */

/*
     8-   (Port B)  
  (BCM - Binary Code Modulation).    BCM
 (256*256)/12000 = 5,46 ms,     183 Hz.
        ,    
        , 
  ,     .
           
       .
*/

ISR (TIMER0_COMPA_vect)
{
	//   
	if (duplex == 0) getPos = bcmPos;		//    
	//    Timer/Counter0
	OCR0A = bitMask[index];					//   
	//   PORTB  
	PORTB = bufferData[index+getPos];		//  BCM 
	//       
	duplex++;								//   1
	duplex &= 15;							//   
	if (duplex <= 7) index = duplex; else index = 15-duplex;	//  
}

/* ------------------------------------------------------------------------- */
/* ---------------------------   -------------------------- */
/* ------------------------------------------------------------------------- */

int main(void)
{
	cli();	//   

	//  :
	//   PB   
	DDRB  = 0xFF;
	//  pull-up      PD6, PD5, PD4, PD1, PD0,    
	PORTD = (1<<PD6) | (1<<PD5) | (1<<PD4) | (1<<PD1) | (1<<PD0);
	/* USB  PD3(D-)  PD2(D+)       pull-up ,
	     V-USB */

	//  
	usbInit();				//  USB 
	usbDeviceDisconnect();	//   USB 
	uchar i = 0;
	while(--i)				//   > 250 
		_delay_ms(1);
	usbDeviceConnect();		//   
	
	//  Timer/Counter0   BCM (.   )
	// WGM[02..00] = 010	(CTC Mode, fOCnx=clk/(N*256) )
	// CS[02..00]  = 100	( N=256)
	// COM0A[1,0]  = 00  	( OC0A/PB2 - )
	// COM0B[1,0]  = 00  	( OC0B/PD5 - )
	// TCCR0A = (COM0A1, COM0A0, COM0B1, COM0B0, -, -, WGM01, WGM00)
	// TCCR0B = (0, 0, -, -, WGM02, CS02, CS01, CS00 )
	TCCR0A = (1<<WGM01);
	TCCR0B = (1<<CS02);
	//      
	OCR0A  = 255;			//    
	TCNT0  = 0;				//   
	//     T0A
	TIMSK |= (1<<OCIE0A);

	// 
	wdt_enable(WDTO_1S);	//       1 
	sei();					//   

	//    main
	while(1)				//  
	{
		wdt_reset();		//    (  )
		usbPoll();			//      (   50 ms)

		if (newData)				//      , 
			newData = bcmMethod();	//      
	}
	return 0;
}
/* ------------------------------------------------------------------------- */
